#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define X_SIZE 192            /* pocet pixelu mikroradku */
#define Y_SIZE 192            /* pocet mikroradku */
#define MAXITEM X_SIZE/2*Y_SIZE /* velikost pole obrazovych dat */
#define MAXVRAM X_SIZE/6*Y_SIZE /* velikost aktivni casti VIDEORAM PMD-85 */
#define BMP_HEADER_LEN 118    /* delka hlavicky souboru BMP */
#define ERR_SIZE -1           /* nepripustna velikost obrazku */
#define ERR_NUMCOL -2         /* nepripustny pocet barev obrazku */
#define ERR_FORMAT -3         /* spatny format souboru (not BMP) */

FILE *vstupni_soubor, *vystupni_soubor, *vystupni_bin_soubor;
char jmeno_vstupniho_souboru[255];
char jmeno_vystupniho_souboru[255];
char jmeno_vystupniho_bin_souboru[255];
unsigned char hlavicka_bmp[BMP_HEADER_LEN];
unsigned char bitmapa[MAXITEM];
unsigned char videoram[MAXVRAM];
int size_x, size_y;

void napoveda(void)
{
    printf("Kompilator fontu pro kartu SAPI/AND-1V\n\n");
    printf("Vstup: BMP 16 barev, rozmer 192x192 pixelu\n");
    printf("Barva pixelu je libovolna mimo cerne.\n");
    printf("Znaky razeny v poctu 32 na radek, celkem 16 radku.\n");
    printf("Zkompiluji se dve sady (alternativni a hlavni) po 256 znacich.\n\n");
    printf("Volani: and1vfont.exe font.bmp\n\n");
    printf("STISKNI ENTER\n\n");
    getchar();
}

/* nacteni hodnoty pixelu z virtualni videoram */
unsigned char get_pixel(int x, int y)
{
  int s_mikroradku;

  s_mikroradku = 4*(1+((size_x-1)/8)); /* interni sirka mikroradku v BMP */

  if (x < size_x)
    if (x == (x/2*2))
    {
      return ((bitmapa[(size_y-1-y)*s_mikroradku+x/2])>>4)&0xf;
    }
    else
    {
      return (bitmapa[(size_y-1-y)*s_mikroradku+x/2])&0xf;
    }
  else
  return 0;  /* mimo platna data vracim cernou */
}

int analyza_dat(int *sizex, int *sizey)
{
  int ukazatel;

  ukazatel = 0;
  while (!feof(vstupni_soubor) & (ukazatel < BMP_HEADER_LEN) ) /* nacteni hlavicky BMP */
    hlavicka_bmp[ukazatel++] = (unsigned char) fgetc(vstupni_soubor);

  ukazatel = 0;
  while (!feof(vstupni_soubor) & (ukazatel < MAXITEM) ) /* nacteni bitmapy BMP */
    bitmapa[ukazatel++] = (unsigned char) fgetc(vstupni_soubor);

  /* nacteni rozmeru obrazku z hlavicky BMP souboru */
  *sizex = (unsigned long)hlavicka_bmp[18]+(unsigned long)256*(unsigned long)hlavicka_bmp[19];
  *sizey = (unsigned long)hlavicka_bmp[22]+(unsigned long)256*(unsigned long)hlavicka_bmp[23];

  if (hlavicka_bmp[0] != 'B') return ERR_FORMAT; /* not metadata BMP */
  if (hlavicka_bmp[1] != 'M') return ERR_FORMAT; /* not metadata BMP */
  if (hlavicka_bmp[28] != 4) return ERR_NUMCOL; /* not 16 barevny BMP */
  if ((*sizex != X_SIZE) || (*sizey != Y_SIZE)) return ERR_SIZE;

  return 0;
}

/* prevod bitmapy na data */
int bmp2data(void)
{
  int chybovy_kod,y,b,size_b,p;
  unsigned char barva;

  chybovy_kod = analyza_dat(&size_x, &size_y);
  if (chybovy_kod) return chybovy_kod;

  size_b = 1 + ((size_x-1)/6); /* pocet bajtu na sirku ve videoram PMD-85 */

  /* vynulovat obraz videoram */

  for (y = 0; y < size_y; y++)
    for (b = 0; b < size_b; b++)
      videoram[(X_SIZE/6)*y+b] = 0;

  printf("Detekovan validni obrazek o rozmerech %i x %i pixelu.\n",size_x,size_y);

  /* vlastni prevod pixelu do obrazu videoram */
  for (y = 0; y < size_y; y++)   /* mikroradky */
    for (b = 0; b < size_b; b++) /* bajty na radku */
    {
      for (p = 0; p < 6; p++)    /* jednotlive bity v bajtu */
      {
        barva = get_pixel(6*b+p,y);
        if (barva) /* kdyz se najde nejaky svitici pixel */
          videoram[(X_SIZE/6)*y+b] |= (1<<(5-p)); /* nahodit pixel */
      }
    }
  return 0;
}

/* zapis obrazu videoram do textove podoby do vystupniho souboru */
/* je pouzito linearni razeni mikroradku */
/* nasobeni x4 je kvuli vyuzitym pinum EEPROM */

void data2txt(void)
{
  unsigned int index,znak_x, znak_y;

  for (znak_y = 0; znak_y < 16; znak_y++)
    for (znak_x = 0; znak_x < 32; znak_x++)
      {
        fprintf(vystupni_soubor,"\t.db ");
        for (index = 0; index < 6; index++)
        {
          fprintf(vystupni_soubor,"%03xh,",4*videoram[((znak_y*12)+index)*(X_SIZE/6)+znak_x]);
          fputc(4*videoram[((znak_y*12)+index)*(X_SIZE/6)+znak_x],vystupni_bin_soubor);
        }
        fprintf(vystupni_soubor,"000h,000h,");
        fputc(0,vystupni_bin_soubor);
        fputc(0,vystupni_bin_soubor);
        for (index = 6; index < 12; index++)
        {
          fprintf(vystupni_soubor,"%03xh,",4*videoram[((znak_y*12)+index)*(X_SIZE/6)+znak_x]);
          fputc(4*videoram[((znak_y*12)+index)*(X_SIZE/6)+znak_x],vystupni_bin_soubor);
        }
        fputc(0,vystupni_bin_soubor);
        fputc(0,vystupni_bin_soubor);
        fprintf(vystupni_soubor,"000h,000h ; %i\n",znak_y*32+znak_x);
      }
}

int main(int argc, char *argv[])
{
    if (argc<2)
      napoveda();
    else
    {
      strcpy(jmeno_vstupniho_souboru,argv[1]);
      strcpy(jmeno_vystupniho_souboru,jmeno_vstupniho_souboru);
      strcpy(jmeno_vystupniho_bin_souboru,jmeno_vstupniho_souboru);
      strcat(jmeno_vystupniho_souboru,".txt");
      strcat(jmeno_vystupniho_bin_souboru,".bin");
      vstupni_soubor = fopen(jmeno_vstupniho_souboru, "rb");
      if (vstupni_soubor == NULL)
        {
          printf("Soubor %s nenalezen!",jmeno_vstupniho_souboru);
        }
      else
        {
          vystupni_soubor = fopen(jmeno_vystupniho_souboru, "wt");
          vystupni_bin_soubor = fopen(jmeno_vystupniho_bin_souboru, "wb");
          if ((vystupni_soubor == NULL)||(vystupni_bin_soubor == NULL))
            {
              printf("Vystupni soubor nelze otevrit!");
            }
          else
            {
              switch (bmp2data())
                {
                  case ERR_SIZE:   printf("Obrazek nema velikost 192x192 pixelu.\n");
                                   printf("Prevod nebude proveden.");
                                   break;
                  case ERR_NUMCOL: printf("Vstupni soubor nema format 16 barevneho BMP.\n");
                                   printf("Prevod nebude proveden.");
                                   break;
                  case ERR_FORMAT: printf("Nepripustny format vstupniho souboru.\n");
                                   printf("Prevod nebude proveden.");
                                   break;
                  case 0:          data2txt();
                                   break;
                  default: printf("Nespecifikovana chyba pri prevodu dat.\n");
                           printf("Prevod nebude proveden.");
                           break;
                }
              fclose(vystupni_soubor);
              fclose(vystupni_bin_soubor);
            }
          fclose(vstupni_soubor);
        }
    printf("\n\nStiskni ENTER\n\n\n");
    getchar();
  }
  return 0;
}
